/*jslint node: true */
/*jslint browser: true*/
/*jslint nomen: true*/
/*jslint plusplus: true */
/*global dashboardApp, angular, $*/

'use strict';

dashboardApp.factory('hostCountWidgetHelper',
    ['$q', 'hostCountService', '$window', '$location',
        function ($q, hostCountService, $window, $location) {
            var helper = {};
            var todayDate = new Date();

            helper.goToHostCountReport = function (daterange) {
                sessionStorage.removeItem('sql');
                var url = MP_SITE_URL + "/advancedreports/goToBootstrappedHostsReport";
                $window.location.href = url;
            }

            helper.chartConfig = {
                chart: {
                    type: 'line',
                    backgroundColor: 'transparent',
                    events: {
                        click: function (e) {
                            helper.goToHostCountReport()
                        }
                    }
                },
                exporting: {
                    enabled: false
                },
                colors: [widgetColors.mainColor()],
                plotOptions: {
                    line: {
                        allowPointSelect: false,
                        cursor: 'pointer',
                        point: {
                            events: {
                                click: function () {
                                    var daterange = {};
                                    daterange.from = moment().startOf('day').subtract((6 - this.x), 'days');
                                    daterange.to = moment().startOf('day').subtract((5 - this.x), 'days').subtract('1', 'milliseconds'); // remove a ms to keep within day
                                    helper.goToHostCountReport(daterange);
                                }
                            }
                        }
                    },
                    series: {
                        borderWidth: 0,
                    }
                },
                tooltip: {
                    formatter: function () {
                        return 'Count: ' + this.y;
                    }
                },
                legend: {
                    enabled: false
                },
                series: [{
                    data: []
                }],
                title: {
                    text: ''
                },
                subtitle: {
                    text: ''
                },
                xAxis: {
                    categories: [],
                    title: {
                        text: null
                    },
                    allowDecimals: false
                },
                yAxis: {
                    allowDecimals: false,
                    min: 0,
                    title: {
                        text: 'Bootstrapped host count'
                    }
                },
                credits: {
                    enabled: false
                },
                loading: false
            };

            helper.getChartConfig = function () {
             return {...helper.chartConfig, colors: [widgetColors.mainColor()]}
            }
            helper.getHostCountStats = function (data, group) {

                var promise = hostCountService.getHostCountStats({
                    from: moment(data.from).format("YYYY-MM-DD 00:00:00"),
                    to: moment(data.to).format("YYYY-MM-DD 23:59:59"),
                    'period': group
                }).then(
                    function (result) {
                        return result;
                    },
                    function (error) {
                        // error
                    }
                );

                return promise
            };


            helper.convertArray = function (arr) {
                var returnData = {};
                returnData.array = [];
                returnData.axis = [];

                angular.forEach(arr, function (value, key) {
                    returnData.axis.push(value.date);
                    returnData.array.push(value.count);
                });

                return returnData;
            };

            helper.setChartData = function (changeStats) {

                var convertedData = helper.convertArray(changeStats);
                helper.chartConfig.series[0] = {
                    data: convertedData.array
                };
                helper.chartConfig.xAxis.categories = convertedData.axis;
            };

            helper.setChartData = function (changeStats) {

                var convertedData = helper.convertArray(changeStats);
                helper.chartConfig.series[0] = {
                    data: convertedData.array
                };
                helper.chartConfig.xAxis.categories = convertedData.axis;
            };

            helper.initDateRange = function () {
                return {
                    startDate: this._returnDate(6),
                    endDate: todayDate,
                    dateLimit: {days: 1},
                    maxDate: todayDate
                };
            };

            helper._returnDate = function (days, startOfDay) {
                var date = new Date(new Date().setDate(todayDate.getDate() - days));

                if (startOfDay === true) {
                    date.setHours(0, 0, 0, 0);
                }

                return date;
            };


            return helper;
        }]);
